@@ -7,7 +7,7 @@ from feathr.definition.anchor import FeatureAnchor
 from feathr.definition.dtype import FeatureType, str_to_value_type, value_type_to_str
 from feathr.definition.feature import Feature
 from feathr.definition.feature_derivations import DerivedFeature
-from feathr.definition.source import HdfsSource, JdbcSource, Source
+from feathr.definition.source import HdfsSource, JdbcSource, Source, SnowflakeSource
 from pyapacheatlas.core import AtlasProcess,AtlasEntity
 
 from feathr.definition.transformation import ExpressionTransformation, Transformation, WindowAggTransformation
@@ -38,7 +38,13 @@ def source_to_def(v: Source) -> dict:
     elif isinstance(v, HdfsSource):
         ret = {
             "name": v.name,
-            "type": urlparse(v.path).scheme,
+            "type": "hdfs",
+            "path": v.path,
+        }
+    elif isinstance(v, SnowflakeSource):
+        ret = {
+            "name": v.name,
+            "type": "SNOWFLAKE",
             "path": v.path,
         }
     elif isinstance(v, JdbcSource):
@@ -59,8 +65,10 @@ def source_to_def(v: Source) -> dict:
         ret["preprocessing"] = inspect.getsource(v.preprocessing)
     if v.event_timestamp_column:
         ret["eventTimestampColumn"] = v.event_timestamp_column
+        ret["event_timestamp_column"] = v.event_timestamp_column
     if v.timestamp_format:
         ret["timestampFormat"] = v.timestamp_format
+        ret["timestamp_format"] = v.timestamp_format
     if v.registry_tags:
         ret["tags"] = v.registry_tags
     return ret
