@@ -1,3 +1,4 @@
+from copy import deepcopy
 from datetime import datetime
 import json
 import os
@@ -10,6 +11,7 @@ from typing import Any, Dict, List, Optional
 from loguru import logger
 from pyspark import *
 
+from feathr.constants import OUTPUT_PATH_TAG
 from feathr.version import get_maven_artifact_fullname
 from feathr.spark_provider._abc import SparkJobLauncher
 
@@ -29,7 +31,7 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
         debug_folder: str = "debug",
         clean_up: bool = True,
         retry: int = 3,
-        retry_sec: int = 5,
+        retry_sec: int = 30,
     ):
         """Initialize the Local Spark job launcher"""
         self.workspace_path = (workspace_path,)
@@ -40,6 +42,7 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
         self.retry_sec = retry_sec
         self.packages = self._get_default_package()
         self.master = master or "local[*]"
+        self.job_tags = None
 
     def upload_or_get_cloud_path(self, local_path_or_http_path: str):
         """For Local Spark Case, no need to upload to cloud workspace."""
@@ -52,6 +55,7 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
         main_class_name: str,
         arguments: List[str] = None,
         python_files: List[str] = None,
+        job_tags: Dict[str, str] = None,
         configuration: Dict[str, str] = {},
         properties: Dict[str, str] = {},
         **_,
@@ -66,9 +70,10 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
             main_class_name: name of your main class
             arguments: all the arguments you want to pass into the spark job
             python_files: required .zip, .egg, or .py files of spark job
+            job_tags: tags of the job, for example you might want to put your user ID, or a tag with a certain information
             configuration: Additional configs for the spark job
             properties: System properties configuration
-            **_: Not used arguments in local spark mode, such as reference_files_path and job_tags
+            **_: Not used arguments in local spark mode, such as reference_files_path
         """
         logger.warning(
             f"Local Spark Mode only support basic params right now and should be used only for testing purpose."
@@ -77,8 +82,11 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
 
         # Get conf and package arguments
         cfg = configuration.copy() if configuration else {}
+        maven_dependency_without_feathr = f"{cfg.pop('spark.jars.packages', self.packages)}"
         maven_dependency = f"{cfg.pop('spark.jars.packages', self.packages)},{get_maven_artifact_fullname()}"
         spark_args = self._init_args(job_name=job_name, confs=cfg)
+        # Add additional repositories
+        spark_args.extend(["--repositories", "https://repository.mulesoft.org/nexus/content/repositories/public/,https://linkedin.jfrog.io/artifactory/open-source/"])
 
         if not main_jar_path:
             # We don't have the main jar, use Maven
@@ -98,10 +106,20 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
                 # This is a PySpark job, no more things to
                 if python_files.__len__() > 1:
                     spark_args.extend(["--py-files", ",".join(python_files[1:])])
-                print(python_files)
+                logger.info(f"Creating python files in {python_files}")
                 spark_args.append(python_files[0])
         else:
-            spark_args.extend(["--class", main_class_name, main_jar_path])
+            if not python_files:
+                # This is a JAR job
+                spark_args.extend(["--class", main_class_name, main_jar_path])
+            else:
+                spark_args.extend(["--jars", main_jar_path])
+                spark_args.extend(["--packages", maven_dependency_without_feathr])
+                # This is a PySpark job, no more things to
+                if python_files.__len__() > 1:
+                    spark_args.extend(["--py-files", ",".join(python_files[1:])])
+                spark_args.append(python_files[0])
+
 
         if arguments:
             spark_args.extend(arguments)
@@ -112,7 +130,9 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
         cmd = " ".join(spark_args)
 
         log_append = open(f"{self.log_path}_{self.spark_job_num}.txt", "a")
-        proc = Popen(split(cmd), shell=False, stdout=log_append, stderr=STDOUT)
+        # remove stderr=STDOUT per https://stackoverflow.com/a/40046887
+        # reference code: https://github.com/lyft/airflow/blob/main/airflow/providers/apache/spark/hooks/spark_submit.py#L391
+        proc = Popen(split(cmd), shell=False, stdout=log_append)
         logger.info(f"Detail job stdout and stderr are in {self.log_path}.")
 
         self.spark_job_num += 1
@@ -125,6 +145,8 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
 
         logger.info(f"Local Spark job submit with pid: {proc.pid}.")
 
+        self.job_tags = deepcopy(job_tags)
+
         return proc
 
     def wait_for_completion(self, timeout_seconds: Optional[float] = 500) -> bool:
@@ -142,6 +164,7 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
         while proc.poll() is None and (((timeout_seconds is None) or (time.time() - start_time < timeout_seconds))):
             time.sleep(1)
             try:
+                last_line = log_read.readlines()[-1]
                 if retry < 1:
                     logger.warning(
                         f"Spark job has hang for {self.retry * self.retry_sec} seconds. latest msg is {last_line}. \
@@ -151,7 +174,6 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
                         self._clean_up()
                         proc.wait()
                     break
-                last_line = log_read.readlines()[-1]
                 retry = self.retry
                 if last_line == []:
                     print("_", end="")
@@ -171,15 +193,33 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
         if proc.returncode == None:
             logger.warning(
                 f"Spark job with pid {self.latest_spark_proc.pid} not completed after {timeout_seconds} sec \
-                    time out setting. Please check."
+                    time out setting. Spark Logs:"
             )
+            with open(log_read.name) as f:
+                contents = f.read()
+                logger.error(contents)
             if self.clean_up:
                 self._clean_up()
                 proc.wait()
                 return True
         elif proc.returncode == 1:
-            logger.warning(f"Spark job with pid {self.latest_spark_proc.pid} is not successful. Please check.")
+            logger.warning(f"Spark job with pid {self.latest_spark_proc.pid} is not successful. Spark Logs:")
+            with open(log_read.name) as f:
+                contents = f.read()
+                logger.error(contents)
             return False
+        elif proc.returncode == 143:
+            # Handle the return code 143 separately.
+            # Normally, the Popen method will return a handle that we can poll, and the poll result will be "None" if it's still running, and will be other values if the subprocess is finished.
+            # However when calling `Popen` with `spark-submit`, for some reason, the poll result will always return "None", and the process will hang there forever
+            # due to this issue, additional handling is needed where we detect the output logs and see if the job is finished.
+            # If the logs gives out hint that the job is finished, even the poll result is None (indicating the process is still running) we will still terminate it.
+            # by calling `proc.terminate()`
+            # if the process is terminated with this way, the return code will be 143. We assume this will still be a successful run.
+            logger.info(
+                f"Spark job with pid {self.latest_spark_proc.pid} finished in: {int(job_duration)} seconds."
+            )
+            return True
         else:
             logger.info(
                 f"Spark job with pid {self.latest_spark_proc.pid} finished in: {int(job_duration)} seconds \
@@ -198,6 +238,22 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
         """Get the status of the job, only a placeholder for local spark"""
         return self.latest_spark_proc.returncode
 
+    def get_job_result_uri(self) -> str:
+        """Get job output path
+
+        Returns:
+            str: output_path
+        """
+        return self.job_tags.get(OUTPUT_PATH_TAG, None) if self.job_tags else None
+
+    def get_job_tags(self) -> Dict[str, str]:
+        """Get job tags
+
+        Returns:
+            Dict[str, str]: a dict of job tags
+        """
+        return self.job_tags
+
     def _init_args(self, job_name: str, confs: Dict[str, str]) -> List[str]:
         logger.info(f"Spark job: {job_name} is running on local spark with master: {self.master}.")
         args = [
@@ -224,7 +280,7 @@ class _FeathrLocalSparkJobLauncher(SparkJobLauncher):
         prefix += datetime.now().strftime("%Y%m%d%H%M%S")
         debug_path = os.path.join(debug_folder, prefix)
 
-        print(debug_path)
+        logger.info(f"Spark log path is {debug_path}")
         if not os.path.exists(debug_path):
             os.makedirs(debug_path)
 
