@@ -10,7 +10,7 @@ from feathr import (BackfillTime, MaterializationSettings)
 from feathr import Feature
 from feathr import FeatureAnchor
 from feathr import FeatureQuery
-from feathr import HdfsSource
+from feathr import HdfsSource, SnowflakeSource
 from feathr import ObservationSettings
 from feathr import RedisSink
 from feathr import STRING, FLOAT, INT32, ValueType
@@ -103,7 +103,7 @@ def test_non_swa_feature_gen_with_offline_preprocessing():
                                            "f_day_of_week"
                                        ],
                                        backfill_time=backfill_time)
-    client.materialize_features(settings)
+    client.materialize_features(settings, allow_materialize_non_agg_feature=True)
     # just assume the job is successful without validating the actual result in Redis. Might need to consolidate
     # this part with the test_feathr_online_store test case
     client.wait_job_to_finish(timeout_sec=Constants.SPARK_JOB_TIMEOUT_SECONDS)
@@ -111,6 +111,8 @@ def test_non_swa_feature_gen_with_offline_preprocessing():
     res = client.get_online_features(online_test_table, '2020-04-01 07:21:51', [
         'f_is_long_trip_distance', 'f_day_of_week'])
     assert res == [8000006.0, 4]
+    
+    client._clean_test_data(online_test_table)
 
 
 def test_feature_swa_feature_gen_with_preprocessing():
@@ -171,6 +173,8 @@ def test_feature_swa_feature_gen_with_preprocessing():
 
     res = client.get_online_features(online_test_table, '265', ['f_location_avg_fare', 'f_location_max_fare'])
     assert res == [1000041.625, 1000100.0]
+    
+    client._clean_test_data(online_test_table)
 
 
 def test_feathr_get_offline_features_hdfs_source():
@@ -402,14 +406,13 @@ def test_feathr_get_offline_features_from_snowflake():
     """
     test_workspace_dir = Path(__file__).parent.resolve() / "test_user_workspace"
     client = snowflake_test_setup(os.path.join(test_workspace_dir, "feathr_config.yaml"))
-    batch_source = HdfsSource(name="nycTaxiBatchSource",
-                              path="jdbc:snowflake://dqllago-ol19457.snowflakecomputing.com/?user=feathrintegration"
-                                   "&sfWarehouse=COMPUTE_WH&dbtable=CALL_CENTER&sfDatabase=SNOWFLAKE_SAMPLE_DATA"
-                                   "&sfSchema=TPCDS_SF10TCL",
-                              preprocessing=snowflake_preprocessing,
-                              event_timestamp_column="lpep_dropoff_datetime",
-                              timestamp_format="yyyy-MM-dd HH:mm:ss")
-
+    batch_source = SnowflakeSource(name="nycTaxiBatchSource",
+                                    database="SNOWFLAKE_SAMPLE_DATA",
+                                    schema="TPCDS_SF10TCL",
+                                    dbtable="CALL_CENTER",
+                                    preprocessing=snowflake_preprocessing,
+                                                            event_timestamp_column="lpep_dropoff_datetime",
+                                                            timestamp_format="yyyy-MM-dd HH:mm:ss")
     call_sk_id = TypedKey(key_column="CC_CALL_CENTER_SK",
                           key_column_type=ValueType.STRING,
                           description="call center sk",
@@ -435,9 +438,10 @@ def test_feathr_get_offline_features_from_snowflake():
     feature_query = FeatureQuery(
         feature_list=['f_snowflake_call_center_division_name_with_preprocessing', 'f_snowflake_call_center_zipcode_with_preprocessing'],
         key=call_sk_id)
+
+    observation_path = client.get_snowflake_path(database="SNOWFLAKE_SAMPLE_DATA", schema="TPCDS_SF10TCL", dbtable="CALL_CENTER")
     settings = ObservationSettings(
-        observation_path='jdbc:snowflake://dqllago-ol19457.snowflakecomputing.com/?user=feathrintegration&sfWarehouse'
-                         '=COMPUTE_WH&dbtable=CALL_CENTER&sfDatabase=SNOWFLAKE_SAMPLE_DATA&sfSchema=TPCDS_SF10TCL')
+        observation_path=observation_path)
 
     now = datetime.now()
     # set output folder based on different runtime
